import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { CreateAdCommand } from './create-ad.command';
import { Inject } from '@nestjs/common';
import { AD_REPOSITORY, PARAMS_PROVIDER } from '@modules/ad/ad.di-tokens';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { Waypoint } from '../../types/waypoint';
import { DefaultParams } from '../../ports/default-params.type';
import { AdRepositoryPort } from '../../ports/ad.repository.port';
import { DefaultParamsProviderPort } from '../../ports/default-params-provider.port';
import { AdAlreadyExistsException } from '@modules/ad/core/domain/ad.errors';
import { AggregateID, ConflictException } from '@mobicoop/ddd-library';

@CommandHandler(CreateAdCommand)
export class CreateAdService implements ICommandHandler {
  private readonly _defaultParams: DefaultParams;

  constructor(
    @Inject(AD_REPOSITORY)
    private readonly repository: AdRepositoryPort,
    @Inject(PARAMS_PROVIDER)
    private readonly defaultParamsProvider: DefaultParamsProviderPort,
  ) {
    this._defaultParams = defaultParamsProvider.getParams();
  }

  async execute(command: CreateAdCommand): Promise<AggregateID> {
    const ad = AdEntity.create(
      {
        userId: command.userId,
        driver: command.driver,
        passenger: command.passenger,
        frequency: command.frequency,
        fromDate: command.fromDate,
        toDate: command.toDate,
        schedule: command.schedule,
        marginDurations: command.marginDurations,
        seatsProposed: command.seatsProposed,
        seatsRequested: command.seatsRequested,
        strict: command.strict,
        waypoints: command.waypoints.map((waypoint: Waypoint) => ({
          position: waypoint.position,
          address: {
            name: waypoint.name,
            houseNumber: waypoint.houseNumber,
            street: waypoint.street,
            postalCode: waypoint.postalCode,
            locality: waypoint.locality,
            country: waypoint.country,
            coordinates: {
              lon: waypoint.lon,
              lat: waypoint.lat,
            },
          },
        })),
      },
      {
        driver: this._defaultParams.DRIVER,
        passenger: this._defaultParams.PASSENGER,
        marginDurations: {
          mon: this._defaultParams.MON_MARGIN,
          tue: this._defaultParams.TUE_MARGIN,
          wed: this._defaultParams.WED_MARGIN,
          thu: this._defaultParams.THU_MARGIN,
          fri: this._defaultParams.FRI_MARGIN,
          sat: this._defaultParams.SAT_MARGIN,
          sun: this._defaultParams.SUN_MARGIN,
        },
        strict: this._defaultParams.STRICT,
        seatsProposed: this._defaultParams.SEATS_PROPOSED,
        seatsRequested: this._defaultParams.SEATS_REQUESTED,
      },
    );

    try {
      await this.repository.insert(ad);
      return ad.id;
    } catch (error: any) {
      if (error instanceof ConflictException) {
        throw new AdAlreadyExistsException(error);
      }
      throw error;
    }
  }
}
